@@ -18,19 +18,32 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 from __future__ import absolute_import
+
+from os.path import isfile
 import re
+
 import mutagen.apev2
 import mutagen.monkeysaudio
 import mutagen.musepack
-import mutagen.wavpack
 import mutagen.optimfrog
-from .mutagenext import tak
-from picard import config, log
-from picard.coverart.image import TagCoverArtImage, CoverArtImageError
+import mutagen.wavpack
+
+from picard import (
+    config,
+    log,
+)
+from picard.coverart.image import (
+    CoverArtImageError,
+    TagCoverArtImage,
+)
 from picard.file import File
 from picard.metadata import Metadata
-from picard.util import encode_filename, sanitize_date
-from os.path import isfile
+from picard.util import (
+    encode_filename,
+    sanitize_date,
+)
+
+from .mutagenext import tak
 
 
 class APEv2File(File):
@@ -49,12 +62,17 @@ class APEv2File(File):
         "Barcode": "barcode",
         "EncodedBy": "encodedby",
         "Language": "language",
+        "MOVEMENT": "movementnumber",
+        "MOVEMENTNAME": "movement",
+        "MOVEMENTTOTAL": "movementtotal",
+        "SHOWMOVEMENT": "showmovement",
         "MUSICBRAINZ_ALBUMSTATUS": "releasestatus",
         "MUSICBRAINZ_ALBUMTYPE": "releasetype",
         "musicbrainz_trackid": "musicbrainz_recordingid",
         "musicbrainz_releasetrackid": "musicbrainz_trackid",
+        "Original Artist": "originalartist",
     }
-    __rtranslate = dict([(v, k) for k, v in __translate.iteritems()])
+    __rtranslate = dict([(v, k) for k, v in __translate.items()])
 
     def _load(self, filename):
         log.debug("Loading file %r", filename)
@@ -63,8 +81,8 @@ class APEv2File(File):
         if file.tags:
             for origname, values in file.tags.items():
                 if origname.lower().startswith("cover art") and values.kind == mutagen.apev2.BINARY:
-                    if '\0' in values.value:
-                        descr, data = values.value.split('\0', 1)
+                    if b'\0' in values.value:
+                        descr, data = values.value.split(b'\0', 1)
                         try:
                             coverartimage = TagCoverArtImage(
                                 file=filename,
@@ -127,20 +145,15 @@ class APEv2File(File):
                     del tags[name]
         temp = {}
         for name, value in metadata.items():
-            if name.startswith("~"):
+            if name.startswith("~") or not self.supports_tag(name):
                 continue
-            if name.startswith('lyrics:'):
-                name = 'Lyrics'
-            elif name == "date":
-                name = "Year"
+            real_name = self._get_tag_name(name)
             # tracknumber/totaltracks => Track
-            elif name == 'tracknumber':
-                name = 'Track'
+            if name == 'tracknumber':
                 if 'totaltracks' in metadata:
                     value = '%s/%s' % (value, metadata['totaltracks'])
             # discnumber/totaldiscs => Disc
             elif name == 'discnumber':
-                name = 'Disc'
                 if 'totaldiscs' in metadata:
                     value = '%s/%s' % (value, metadata['totaldiscs'])
             elif name in ('totaltracks', 'totaldiscs'):
@@ -148,45 +161,40 @@ class APEv2File(File):
             # "performer:Piano=Joe Barr" => "Performer=Joe Barr (Piano)"
             elif name.startswith('performer:') or name.startswith('comment:'):
                 name, desc = name.split(':', 1)
-                name = name.title()
                 if desc:
                     value += ' (%s)' % desc
-            elif name in self.__rtranslate:
-                name = self.__rtranslate[name]
-            else:
-                name = name.title()
-            temp.setdefault(name, []).append(value)
+            temp.setdefault(real_name, []).append(value)
         for name, values in temp.items():
-            tags[str(name)] = values
+            tags[name] = values
         for image in metadata.images_to_be_saved_to_tags:
             cover_filename = 'Cover Art (Front)'
             cover_filename += image.extension
-            tags['Cover Art (Front)'] = mutagen.apev2.APEValue(cover_filename + '\0' + image.data, mutagen.apev2.BINARY)
+            tags['Cover Art (Front)'] = mutagen.apev2.APEValue(cover_filename.encode('ascii') + b'\0' + image.data, mutagen.apev2.BINARY)
             break
             # can't save more than one item with the same name
             # (mp3tags does this, but it's against the specs)
 
         self._remove_deleted_tags(metadata, tags)
-
         tags.save(encode_filename(filename))
 
     def _remove_deleted_tags(self, metadata, tags):
         """Remove the tags from the file that were deleted in the UI"""
         for tag in metadata.deleted_tags:
-            real_name = str(self._get_tag_name(tag))
+            real_name = self._get_tag_name(tag)
             if real_name in ('Lyrics', 'Comment', 'Performer'):
-                tag_type = r"\(%s\)" % tag.split(':', 1)[1]
-                for item in tags.get(real_name):
-                    if re.search(tag_type, item):
-                        tags.get(real_name).remove(item)
+                tag_type = re.compile(r"\(%s\)" % tag.split(':', 1)[1])
+                existing_tags = tags.get(real_name)
+                if existing_tags:
+                    for item in existing_tags:
+                        if tag_type.search(item):
+                            tags.get(real_name).remove(item)
             elif tag in ('totaltracks', 'totaldiscs'):
                 tagstr = real_name.lower() + 'number'
-                try:
+                if tagstr in metadata:
                     tags[real_name] = metadata[tagstr]
-                except KeyError:
-                    pass
             else:
-                del tags[real_name]
+                if real_name in tags:
+                    del tags[real_name]
 
     def _get_tag_name(self, name):
         if name.startswith('lyrics:'):
@@ -204,8 +212,17 @@ class APEv2File(File):
         else:
             return name.title()
 
-    def supports_tag(self, name):
-        return bool(name)
+    @classmethod
+    def supports_tag(cls, name):
+        unsupported_tags = {
+            'gapless',
+            'musicip_fingerprint',
+            'podcast',
+            'podcasturl',
+            'show',
+            'showsort',
+        }
+        return bool(name) and name not in unsupported_tags
 
 
 class MusepackFile(APEv2File):
@@ -216,7 +233,7 @@ class MusepackFile(APEv2File):
     _File = mutagen.musepack.Musepack
 
     def _info(self, metadata, file):
-        super(MusepackFile, self)._info(metadata, file)
+        super()._info(metadata, file)
         metadata['~format'] = "Musepack, SV%d" % file.info.version
 
 
@@ -227,10 +244,6 @@ class WavPackFile(APEv2File):
     NAME = "WavPack"
     _File = mutagen.wavpack.WavPack
 
-    def _info(self, metadata, file):
-        super(WavPackFile, self)._info(metadata, file)
-        metadata['~format'] = self.NAME
-
     def _save_and_rename(self, old_filename, metadata):
         """Includes an additional check for WavPack correction files"""
         wvc_filename = old_filename.replace(".wv", ".wvc")
@@ -248,8 +261,12 @@ class OptimFROGFile(APEv2File):
     _File = mutagen.optimfrog.OptimFROG
 
     def _info(self, metadata, file):
-        super(OptimFROGFile, self)._info(metadata, file)
-        if file.filename.lower().endswith(".ofs"):
+        super()._info(metadata, file)
+        # mutagen.File.filename can be either a bytes or str object
+        filename = file.filename
+        if isinstance(filename, bytes):
+            filename = filename.decode()
+        if filename.lower().endswith(".ofs"):
             metadata['~format'] = "OptimFROG DualStream Audio"
         else:
             metadata['~format'] = "OptimFROG Lossless Audio"
@@ -262,10 +279,6 @@ class MonkeysAudioFile(APEv2File):
     NAME = "Monkey's Audio"
     _File = mutagen.monkeysaudio.MonkeysAudio
 
-    def _info(self, metadata, file):
-        super(MonkeysAudioFile, self)._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class TAKFile(APEv2File):
 
@@ -273,7 +286,3 @@ class TAKFile(APEv2File):
     EXTENSIONS = [".tak"]
     NAME = "Tom's lossless Audio Kompressor"
     _File = tak.TAK
-
-    def _info(self, metadata, file):
-        super(TAKFile, self)._info(metadata, file)
-        metadata['~format'] = self.NAME
