@@ -59,9 +59,11 @@ def save_and_load_metadata(filename, metadata):
     loaded_metadata = load_metadata(filename)
     return loaded_metadata
 
+
 def load_raw(filename):
     return mutagen.File(filename)
 
+
 TAGS = {
     'albumartist': 'Foo',
     'albumartistsort': 'Foo',
@@ -763,40 +765,51 @@ class TestCoverArt(PicardTestCase):
             os.path.join('test', 'data', 'test.flac'),
             set('a'))
 
+    def test_flac_set_picture_dimensions(self):
+        self._set_up(os.path.join('test', 'data', 'test.flac'))
+        try:
+            tests = [
+                CoverArtImage(data=self.jpegdata),
+                CoverArtImage(data=self.pngdata),
+            ]
+            for test in tests:
+                self._file_save_image(self.filename, test)
+                raw_metadata = load_raw(self.filename)
+                pic = raw_metadata.pictures[0]
+                self.assertNotEqual(pic.width, 0)
+                self.assertEqual(pic.width, test.width)
+                self.assertNotEqual(pic.height, 0)
+                self.assertEqual(pic.height, test.height)
+        finally:
+            self._tear_down()
+
     def _test_cover_art(self, filename):
         self._set_up(filename)
         try:
+            source_types = ["front", "booklet"]
             # Use reasonable large data > 64kb.
             # This checks a mutagen error with ASF files.
-            tests = {
-                'jpg': {
-                    'mime': 'image/jpeg',
-                    'data': self.jpegdata + b"a" * 1024 * 128
-                },
-                'png': {
-                    'mime': 'image/png',
-                    'data': self.pngdata + b"a" * 1024 * 128
-                },
-            }
-            for t in tests:
-                f = picard.formats.open_(self.filename)
-                metadata = Metadata()
-                imgdata = tests[t]['data']
-                metadata.append_image(
-                    CoverArtImage(
-                        data=imgdata
-                    )
-                )
-                f._save(self.filename, metadata)
-
+            tests = [
+                CoverArtImage(data=self.jpegdata + b"a" * 1024 * 128, types=source_types),
+                CoverArtImage(data=self.pngdata + b"a" * 1024 * 128, types=source_types),
+            ]
+            for test in tests:
+                self._file_save_image(self.filename, test)
                 f = picard.formats.open_(self.filename)
                 loaded_metadata = f._load(self.filename)
                 image = loaded_metadata.images[0]
-                self.assertEqual(image.mimetype, tests[t]['mime'])
-                self.assertEqual(image.data, imgdata)
+                self.assertEqual(test.mimetype, image.mimetype)
+                self.assertEqual(test, image)
         finally:
             self._tear_down()
 
+    @staticmethod
+    def _file_save_image(filename, image):
+        f = picard.formats.open_(filename)
+        metadata = Metadata()
+        metadata.append_image(image)
+        f._save(filename, metadata)
+
     def _cover_metadata(self):
         imgdata = self.jpegdata
         metadata = Metadata()
